#######################################################
# 
# UserTableController.py
# Python implementation of the Class UserTableController
# Generated by Enterprise Architect
# Created on:      24-Sep-2020 5:12:31 PM
# Original author: natha
# 
#######################################################
from FreeTAKServer.controllers.DatabaseControllers.TableController import TableController
from FreeTAKServer.model.SQLAlchemy.User import User

class UserTableController(TableController):
    def __init__(self):
        self.table = User